@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -120,7 +118,7 @@ class IssueType:
         self.source = source
         self.references = references or []  # type: ignore
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_attribute()
     def type(self) -> IssueClassification:
         """
@@ -135,7 +133,7 @@ class IssueType:
     def type(self, type: IssueClassification) -> None:
         self._type = type
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def id(self) -> Optional[str]:
         """
@@ -150,7 +148,7 @@ class IssueType:
     def id(self, id: Optional[str]) -> None:
         self._id = id
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(2)
     def name(self) -> Optional[str]:
         """
@@ -165,7 +163,7 @@ class IssueType:
     def name(self, name: Optional[str]) -> None:
         self._name = name
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(3)
     def description(self) -> Optional[str]:
         """
@@ -180,7 +178,7 @@ class IssueType:
     def description(self, description: Optional[str]) -> None:
         self._description = description
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(4)
     def source(self) -> Optional[IssueTypeSource]:
         """
@@ -195,10 +193,10 @@ class IssueType:
     def source(self, source: Optional[IssueTypeSource]) -> None:
         self._source = source
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'url')
     @serializable.xml_sequence(5)
-    def references(self) -> "SortedSet[XsUri]":
+    def references(self) -> 'SortedSet[XsUri]':
         """
         Any reference URLs related to this issue.
 
