@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # This file is part of CycloneDX Python Lib
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,12 +18,14 @@
 from datetime import datetime
 from typing import Iterable, Optional
 
+import serializable
 from sortedcontainers import SortedSet
 
 from ..model import Note, Property, XsUri
 from ..model.issue import IssueType
 
 
+@serializable.serializable_class
 class ReleaseNotes:
     """
     This is our internal representation of a `releaseNotesType` for a Component in a BOM.
@@ -34,12 +34,12 @@ class ReleaseNotes:
         See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.4/#type_releaseNotesType
     """
 
-    def __init__(self, *, type_: str, title: Optional[str] = None, featured_image: Optional[XsUri] = None,
+    def __init__(self, *, type: str, title: Optional[str] = None, featured_image: Optional[XsUri] = None,
                  social_image: Optional[XsUri] = None, description: Optional[str] = None,
                  timestamp: Optional[datetime] = None, aliases: Optional[Iterable[str]] = None,
                  tags: Optional[Iterable[str]] = None, resolves: Optional[Iterable[IssueType]] = None,
                  notes: Optional[Iterable[Note]] = None, properties: Optional[Iterable[Property]] = None) -> None:
-        self.type = type_
+        self.type = type
         self.title = title
         self.featured_image = featured_image
         self.social_image = social_image
@@ -52,6 +52,7 @@ class ReleaseNotes:
         self.properties = properties or []  # type: ignore
 
     @property
+    @serializable.xml_sequence(1)
     def type(self) -> str:
         """
         The software versioning type.
@@ -73,10 +74,11 @@ class ReleaseNotes:
         return self._type
 
     @type.setter
-    def type(self, type_: str) -> None:
-        self._type = type_
+    def type(self, type: str) -> None:
+        self._type = type
 
     @property
+    @serializable.xml_sequence(2)
     def title(self) -> Optional[str]:
         """
         The title of the release.
@@ -88,6 +90,7 @@ class ReleaseNotes:
         self._title = title
 
     @property
+    @serializable.xml_sequence(3)
     def featured_image(self) -> Optional[XsUri]:
         """
         The URL to an image that may be prominently displayed with the release note.
@@ -99,6 +102,7 @@ class ReleaseNotes:
         self._featured_image = featured_image
 
     @property
+    @serializable.xml_sequence(4)
     def social_image(self) -> Optional[XsUri]:
         """
         The URL to an image that may be used in messaging on social media platforms.
@@ -110,6 +114,7 @@ class ReleaseNotes:
         self._social_image = social_image
 
     @property
+    @serializable.xml_sequence(5)
     def description(self) -> Optional[str]:
         """
         A short description of the release.
@@ -121,6 +126,8 @@ class ReleaseNotes:
         self._description = description
 
     @property
+    @serializable.type_mapping(serializable.helpers.XsdDateTime)
+    @serializable.xml_sequence(6)
     def timestamp(self) -> Optional[datetime]:
         """
         The date and time (timestamp) when the release note was created.
@@ -132,7 +139,9 @@ class ReleaseNotes:
         self._timestamp = timestamp
 
     @property
-    def aliases(self) -> "SortedSet[str]":
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'alias')
+    @serializable.xml_sequence(7)
+    def aliases(self) -> 'SortedSet[str]':
         """
         One or more alternate names the release may be referred to. This may include unofficial terms used by
         development and marketing teams (e.g. code names).
@@ -147,7 +156,9 @@ class ReleaseNotes:
         self._aliases = SortedSet(aliases)
 
     @property
-    def tags(self) -> "SortedSet[str]":
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'tag')
+    @serializable.xml_sequence(8)
+    def tags(self) -> 'SortedSet[str]':
         """
         One or more tags that may aid in search or retrieval of the release note.
 
@@ -161,7 +172,9 @@ class ReleaseNotes:
         self._tags = SortedSet(tags)
 
     @property
-    def resolves(self) -> "SortedSet[IssueType]":
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'issue')
+    @serializable.xml_sequence(9)
+    def resolves(self) -> 'SortedSet[IssueType]':
         """
         A collection of issues that have been resolved.
 
@@ -175,7 +188,9 @@ class ReleaseNotes:
         self._resolves = SortedSet(resolves)
 
     @property
-    def notes(self) -> "SortedSet[Note]":
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'note')
+    @serializable.xml_sequence(10)
+    def notes(self) -> 'SortedSet[Note]':
         """
         Zero or more release notes containing the locale and content. Multiple note elements may be specified to support
         release notes in a wide variety of languages.
@@ -190,7 +205,9 @@ class ReleaseNotes:
         self._notes = SortedSet(notes)
 
     @property
-    def properties(self) -> "SortedSet[Property]":
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'property')
+    @serializable.xml_sequence(11)
+    def properties(self) -> 'SortedSet[Property]':
         """
         Provides the ability to document properties in a name-value store. This provides flexibility to include data not
         officially supported in the standard without having to use additional namespaces or create extensions. Unlike
