@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -15,20 +13,18 @@
 # SPDX-License-Identifier: Apache-2.0
 # Copyright (c) OWASP Foundation. All Rights Reserved.
 
-from typing import Any, Iterable, Optional
+from typing import Any, Iterable, Optional, Union
 
+import serializable
 from sortedcontainers import SortedSet
 
-from . import (
-    ComparableTuple,
-    DataClassification,
-    ExternalReference,
-    LicenseChoice,
-    OrganizationalEntity,
-    Property,
-    XsUri,
-)
+from cyclonedx.serialization import BomRefHelper, LicenseRepositoryHelper
+
+from ..schema.schema import SchemaVersion1Dot3, SchemaVersion1Dot4
+from . import ComparableTuple, DataClassification, ExternalReference, OrganizationalEntity, Property, XsUri
 from .bom_ref import BomRef
+from .dependency import Dependable
+from .license import License, LicenseRepository
 from .release_note import ReleaseNotes
 
 """
@@ -39,7 +35,8 @@ This set of classes represents the data that is possible about known Services.
 """
 
 
-class Service:
+@serializable.serializable_class
+class Service(Dependable):
     """
     Class that models the `service` complex type in the CycloneDX schema.
 
@@ -47,17 +44,21 @@ class Service:
         See the CycloneDX schema: https://cyclonedx.org/docs/1.4/xml/#type_service
     """
 
-    def __init__(self, *, name: str, bom_ref: Optional[str] = None, provider: Optional[OrganizationalEntity] = None,
+    def __init__(self, *, name: str, bom_ref: Optional[Union[str, BomRef]] = None,
+                 provider: Optional[OrganizationalEntity] = None,
                  group: Optional[str] = None, version: Optional[str] = None, description: Optional[str] = None,
                  endpoints: Optional[Iterable[XsUri]] = None, authenticated: Optional[bool] = None,
                  x_trust_boundary: Optional[bool] = None, data: Optional[Iterable[DataClassification]] = None,
-                 licenses: Optional[Iterable[LicenseChoice]] = None,
+                 licenses: Optional[Iterable[License]] = None,
                  external_references: Optional[Iterable[ExternalReference]] = None,
                  properties: Optional[Iterable[Property]] = None,
                  services: Optional[Iterable['Service']] = None,
                  release_notes: Optional[ReleaseNotes] = None,
                  ) -> None:
-        self._bom_ref = BomRef(value=bom_ref)
+        if isinstance(bom_ref, BomRef):
+            self._bom_ref = bom_ref
+        else:
+            self._bom_ref = BomRef(value=str(bom_ref) if bom_ref else None)
         self.provider = provider
         self.group = group
         self.name = name
@@ -74,6 +75,10 @@ class Service:
         self.properties = properties or []  # type: ignore
 
     @property
+    @serializable.json_name('bom-ref')
+    @serializable.type_mapping(BomRefHelper)
+    @serializable.xml_attribute()
+    @serializable.xml_name('bom-ref')
     def bom_ref(self) -> BomRef:
         """
         An optional identifier which can be used to reference the service elsewhere in the BOM. Uniqueness is enforced
@@ -87,6 +92,7 @@ class Service:
         return self._bom_ref
 
     @property
+    @serializable.xml_sequence(1)
     def provider(self) -> Optional[OrganizationalEntity]:
         """
         Get the The organization that provides the service.
@@ -101,6 +107,7 @@ class Service:
         self._provider = provider
 
     @property
+    @serializable.xml_sequence(2)
     def group(self) -> Optional[str]:
         """
         The grouping name, namespace, or identifier. This will often be a shortened, single name of the company or
@@ -116,6 +123,7 @@ class Service:
         self._group = group
 
     @property
+    @serializable.xml_sequence(3)
     def name(self) -> str:
         """
         The name of the service. This will often be a shortened, single name of the service.
@@ -130,6 +138,7 @@ class Service:
         self._name = name
 
     @property
+    @serializable.xml_sequence(4)
     def version(self) -> Optional[str]:
         """
         The service version.
@@ -144,6 +153,7 @@ class Service:
         self._version = version
 
     @property
+    @serializable.xml_sequence(5)
     def description(self) -> Optional[str]:
         """
         Specifies a description for the service.
@@ -158,7 +168,9 @@ class Service:
         self._description = description
 
     @property
-    def endpoints(self) -> "SortedSet[XsUri]":
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'endpoint')
+    @serializable.xml_sequence(6)
+    def endpoints(self) -> 'SortedSet[XsUri]':
         """
         A list of endpoints URI's this service provides.
 
@@ -172,6 +184,7 @@ class Service:
         self._endpoints = SortedSet(endpoints)
 
     @property
+    @serializable.xml_sequence(7)
     def authenticated(self) -> Optional[bool]:
         """
         A boolean value indicating if the service requires authentication. A value of true indicates the service
@@ -189,6 +202,9 @@ class Service:
         self._authenticated = authenticated
 
     @property
+    @serializable.json_name('x-trust-boundary')
+    @serializable.xml_name('x-trust-boundary')
+    @serializable.xml_sequence(8)
     def x_trust_boundary(self) -> Optional[bool]:
         """
         A boolean value indicating if use of the service crosses a trust zone or boundary. A value of true indicates
@@ -206,7 +222,9 @@ class Service:
         self._x_trust_boundary = x_trust_boundary
 
     @property
-    def data(self) -> "SortedSet[DataClassification]":
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'classification')
+    @serializable.xml_sequence(9)
+    def data(self) -> 'SortedSet[DataClassification]':
         """
         Specifies the data classification.
 
@@ -220,7 +238,9 @@ class Service:
         self._data = SortedSet(data)
 
     @property
-    def licenses(self) -> "SortedSet[LicenseChoice]":
+    @serializable.type_mapping(LicenseRepositoryHelper)
+    @serializable.xml_sequence(10)
+    def licenses(self) -> LicenseRepository:
         """
         A optional list of statements about how this Service is licensed.
 
@@ -230,11 +250,13 @@ class Service:
         return self._licenses
 
     @licenses.setter
-    def licenses(self, licenses: Iterable[LicenseChoice]) -> None:
-        self._licenses = SortedSet(licenses)
+    def licenses(self, licenses: Iterable[License]) -> None:
+        self._licenses = LicenseRepository(licenses)
 
     @property
-    def external_references(self) -> "SortedSet[ExternalReference]":
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'reference')
+    @serializable.xml_sequence(11)
+    def external_references(self) -> 'SortedSet[ExternalReference]':
         """
         Provides the ability to document external references related to the Service.
 
@@ -248,6 +270,8 @@ class Service:
         self._external_references = SortedSet(external_references)
 
     @property
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'service')
+    @serializable.xml_sequence(13)
     def services(self) -> "SortedSet['Service']":
         """
         A list of services included or deployed behind the parent service.
@@ -266,6 +290,8 @@ class Service:
         self._services = SortedSet(services)
 
     @property
+    @serializable.view(SchemaVersion1Dot4)
+    @serializable.xml_sequence(14)
     def release_notes(self) -> Optional[ReleaseNotes]:
         """
         Specifies optional release notes.
@@ -280,7 +306,11 @@ class Service:
         self._release_notes = release_notes
 
     @property
-    def properties(self) -> "SortedSet[Property]":
+    @serializable.view(SchemaVersion1Dot3)
+    @serializable.view(SchemaVersion1Dot4)
+    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'property')
+    @serializable.xml_sequence(12)
+    def properties(self) -> 'SortedSet[Property]':
         """
         Provides the ability to document properties in a key/value store. This provides flexibility to include data not
         officially supported in the standard without having to use additional namespaces or create extensions.
@@ -313,4 +343,4 @@ class Service:
         ))
 
     def __repr__(self) -> str:
-        return f'<Service group={self.group}, name={self.name}, version={self.version}, bom-ref={self.bom_ref}>'
+        return f'<Service bom-ref={self.bom_ref}, group={self.group}, name={self.name}, version={self.version}>'
