@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # This file is part of CycloneDX Python Lib
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -23,7 +21,6 @@ from datetime import datetime
 from decimal import Decimal
 from enum import Enum
 from typing import Any, Iterable, Optional, Tuple, Union
-from uuid import uuid4
 
 import serializable
 from sortedcontainers import SortedSet
@@ -77,7 +74,7 @@ class BomTargetVersionRange:
         self.range = range
         self.status = status
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def version(self) -> Optional[str]:
         """
@@ -89,7 +86,7 @@ class BomTargetVersionRange:
     def version(self, version: Optional[str]) -> None:
         self._version = version
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(2)
     def range(self) -> Optional[str]:
         """
@@ -106,12 +103,12 @@ class BomTargetVersionRange:
     def range(self, range: Optional[str]) -> None:
         self._range = range
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(3)
     def status(self) -> Optional[ImpactAnalysisAffectedStatus]:
         """
         The vulnerability status for the version or range of versions.
-        """""
+        """
         return self._status
 
     @status.setter
@@ -154,7 +151,7 @@ class BomTarget:
         self.ref = ref
         self.versions = versions or []  # type: ignore
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def ref(self) -> str:
         """
@@ -166,10 +163,10 @@ class BomTarget:
     def ref(self, ref: str) -> None:
         self._ref = ref
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'version')
     @serializable.xml_sequence(2)
-    def versions(self) -> "SortedSet[BomTargetVersionRange]":
+    def versions(self) -> 'SortedSet[BomTargetVersionRange]':
         """
         Zero or more individual versions or range of versions.
 
@@ -222,7 +219,7 @@ class VulnerabilityAnalysis:
         self.responses = responses or []  # type: ignore
         self.detail = detail
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def state(self) -> Optional[ImpactAnalysisState]:
         """
@@ -237,7 +234,7 @@ class VulnerabilityAnalysis:
     def state(self, state: Optional[ImpactAnalysisState]) -> None:
         self._state = state
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(2)
     def justification(self) -> Optional[ImpactAnalysisJustification]:
         """
@@ -252,11 +249,11 @@ class VulnerabilityAnalysis:
     def justification(self, justification: Optional[ImpactAnalysisJustification]) -> None:
         self._justification = justification
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.json_name('response')
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'response')
     @serializable.xml_sequence(3)
-    def responses(self) -> "SortedSet[ImpactAnalysisResponse]":
+    def responses(self) -> 'SortedSet[ImpactAnalysisResponse]':
         """
         A list of responses to the vulnerability by the manufacturer, supplier, or project responsible for the
         affected component or service. More than one response is allowed. Responses are strongly encouraged for
@@ -271,7 +268,7 @@ class VulnerabilityAnalysis:
     def responses(self, responses: Iterable[ImpactAnalysisResponse]) -> None:
         self._responses = SortedSet(responses)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(4)
     def detail(self) -> Optional[str]:
         """
@@ -313,7 +310,7 @@ class VulnerabilityAdvisory:
         self.title = title
         self.url = url
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def title(self) -> Optional[str]:
         """
@@ -325,7 +322,7 @@ class VulnerabilityAdvisory:
     def title(self, title: Optional[str]) -> None:
         self._title = title
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(2)
     def url(self) -> XsUri:
         """
@@ -373,7 +370,7 @@ class VulnerabilitySource:
         self.name = name
         self.url = url
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def name(self) -> Optional[str]:
         """
@@ -385,7 +382,7 @@ class VulnerabilitySource:
     def name(self, name: Optional[str]) -> None:
         self._name = name
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(2)
     def url(self) -> Optional[XsUri]:
         """
@@ -436,7 +433,7 @@ class VulnerabilityReference:
         self.id = id
         self.source = source
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def id(self) -> Optional[str]:
         """
@@ -448,7 +445,7 @@ class VulnerabilityReference:
     def id(self, id: Optional[str]) -> None:
         self._id = id
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(2)
     def source(self) -> Optional[VulnerabilitySource]:
         """
@@ -563,7 +560,7 @@ class VulnerabilitySeverity(str, Enum):
     UNKNOWN = 'unknown'
 
     @staticmethod
-    def get_from_cvss_scores(scores: Union[Tuple[float], float, None]) -> 'VulnerabilitySeverity':
+    def get_from_cvss_scores(scores: Union[Tuple[float, ...], float, None]) -> 'VulnerabilitySeverity':
         """
         Derives the Severity of a Vulnerability from it's declared CVSS scores.
 
@@ -634,17 +631,18 @@ class VulnerabilityRating:
         self.justification = justification
 
         if score_base:
-            warnings.warn('`score_base` is deprecated - use `score`', DeprecationWarning)
+            warnings.warn('`score_base` is deprecated - use `score`',
+                          category=DeprecationWarning, stacklevel=1)
             if score:
                 warnings.warn('Both `score` and `score_base` supplied - the deprecated `score_base` will be discarded',
-                              DeprecationWarning)
+                              category=DeprecationWarning, stacklevel=1)
             else:
                 self.score = Decimal(score_base)
 
         if vector and method:
             self.vector = method.get_localised_vector(vector=vector)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def source(self) -> Optional[VulnerabilitySource]:
         """
@@ -656,7 +654,7 @@ class VulnerabilityRating:
     def source(self, source: Optional[VulnerabilitySource]) -> None:
         self._source = source
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.string_format('.1f')
     @serializable.xml_sequence(2)
     def score(self) -> Optional[Decimal]:
@@ -669,7 +667,7 @@ class VulnerabilityRating:
     def score(self, score: Optional[Decimal]) -> None:
         self._score = score
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(3)
     def severity(self) -> Optional[VulnerabilitySeverity]:
         """
@@ -681,7 +679,7 @@ class VulnerabilityRating:
     def severity(self, severity: Optional[VulnerabilitySeverity]) -> None:
         self._severity = severity
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(4)
     def method(self) -> Optional[VulnerabilityScoreSource]:
         """
@@ -693,7 +691,7 @@ class VulnerabilityRating:
     def method(self, score_source: Optional[VulnerabilityScoreSource]) -> None:
         self._method = score_source
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(5)
     def vector(self) -> Optional[str]:
         """
@@ -705,7 +703,7 @@ class VulnerabilityRating:
     def vector(self, vector: Optional[str]) -> None:
         self._vector = vector
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(6)
     def justification(self) -> Optional[str]:
         """
@@ -761,10 +759,10 @@ class VulnerabilityCredits:
         self.organizations = organizations or []  # type: ignore
         self.individuals = individuals or []  # type: ignore
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'organization')
     @serializable.xml_sequence(1)
-    def organizations(self) -> "SortedSet[OrganizationalEntity]":
+    def organizations(self) -> 'SortedSet[OrganizationalEntity]':
         """
         The organizations credited with vulnerability discovery.
 
@@ -777,10 +775,10 @@ class VulnerabilityCredits:
     def organizations(self, organizations: Iterable[OrganizationalEntity]) -> None:
         self._organizations = SortedSet(organizations)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'individual')
     @serializable.xml_sequence(2)
-    def individuals(self) -> "SortedSet[OrganizationalContact]":
+    def individuals(self) -> 'SortedSet[OrganizationalContact]':
         """
         The individuals, not associated with organizations, that are credited with vulnerability discovery.
 
@@ -836,10 +834,10 @@ class Vulnerability:
                  # Deprecated Parameters kept for backwards compatibility
                  source_name: Optional[str] = None, source_url: Optional[str] = None,
                  recommendations: Optional[Iterable[str]] = None) -> None:
-        if type(bom_ref) == BomRef:
+        if isinstance(bom_ref, BomRef):
             self._bom_ref = bom_ref
         else:
-            self._bom_ref = BomRef(value=str(bom_ref) if bom_ref else str(uuid4()))
+            self._bom_ref = BomRef(value=str(bom_ref) if bom_ref else None)
         self.id = id
         self.source = source
         self.references = references or []  # type: ignore
@@ -859,16 +857,18 @@ class Vulnerability:
         self.properties = properties or []  # type: ignore
 
         if source_name or source_url:
-            warnings.warn('`source_name` and `source_url` are deprecated - use `source`', DeprecationWarning)
+            warnings.warn('`source_name` and `source_url` are deprecated - use `source`',
+                          category=DeprecationWarning, stacklevel=1)
             if not source:
                 self.source = VulnerabilitySource(name=source_name, url=XsUri(source_url) if source_url else None)
 
         if recommendations:
-            warnings.warn('`recommendations` is deprecated - use `recommendation`', DeprecationWarning)
+            warnings.warn('`recommendations` is deprecated - use `recommendation`',
+                          category=DeprecationWarning, stacklevel=1)
             if not recommendation:
                 self.recommendation = next(iter(recommendations))
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.json_name('bom-ref')
     @serializable.type_mapping(BomRefHelper)
     @serializable.xml_attribute()
@@ -884,7 +884,7 @@ class Vulnerability:
         """
         return self._bom_ref
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(1)
     def id(self) -> Optional[str]:
         """
@@ -899,7 +899,7 @@ class Vulnerability:
     def id(self, id: Optional[str]) -> None:
         self._id = id
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(2)
     def source(self) -> Optional[VulnerabilitySource]:
         """
@@ -914,10 +914,10 @@ class Vulnerability:
     def source(self, source: Optional[VulnerabilitySource]) -> None:
         self._source = source
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'reference')
     @serializable.xml_sequence(3)
-    def references(self) -> "SortedSet[VulnerabilityReference]":
+    def references(self) -> 'SortedSet[VulnerabilityReference]':
         """
         Zero or more pointers to vulnerabilities that are the equivalent of the vulnerability specified. Often times,
         the same vulnerability may exist in multiple sources of vulnerability intelligence, but have different
@@ -933,10 +933,10 @@ class Vulnerability:
     def references(self, references: Iterable[VulnerabilityReference]) -> None:
         self._references = SortedSet(references)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'rating')
     @serializable.xml_sequence(4)
-    def ratings(self) -> "SortedSet[VulnerabilityRating]":
+    def ratings(self) -> 'SortedSet[VulnerabilityRating]':
         """
         List of vulnerability ratings.
 
@@ -949,10 +949,10 @@ class Vulnerability:
     def ratings(self, ratings: Iterable[VulnerabilityRating]) -> None:
         self._ratings = SortedSet(ratings)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'cwe')
     @serializable.xml_sequence(5)
-    def cwes(self) -> "SortedSet[int]":
+    def cwes(self) -> 'SortedSet[int]':
         """
         A list of CWE (Common Weakness Enumeration) identifiers.
 
@@ -968,7 +968,7 @@ class Vulnerability:
     def cwes(self, cwes: Iterable[int]) -> None:
         self._cwes = SortedSet(cwes)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(6)
     def description(self) -> Optional[str]:
         """
@@ -983,7 +983,7 @@ class Vulnerability:
     def description(self, description: Optional[str]) -> None:
         self._description = description
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(7)
     def detail(self) -> Optional[str]:
         """
@@ -999,7 +999,7 @@ class Vulnerability:
     def detail(self, detail: Optional[str]) -> None:
         self._detail = detail
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(8)
     def recommendation(self) -> Optional[str]:
         """
@@ -1014,10 +1014,10 @@ class Vulnerability:
     def recommendation(self, recommendation: Optional[str]) -> None:
         self._recommendation = recommendation
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'advisory')
     @serializable.xml_sequence(9)
-    def advisories(self) -> "SortedSet[VulnerabilityAdvisory]":
+    def advisories(self) -> 'SortedSet[VulnerabilityAdvisory]':
         """
         Advisories relating to the Vulnerability.
 
@@ -1030,7 +1030,7 @@ class Vulnerability:
     def advisories(self, advisories: Iterable[VulnerabilityAdvisory]) -> None:
         self._advisories = SortedSet(advisories)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.type_mapping(serializable.helpers.XsdDateTime)
     @serializable.xml_sequence(10)
     def created(self) -> Optional[datetime]:
@@ -1046,7 +1046,7 @@ class Vulnerability:
     def created(self, created: Optional[datetime]) -> None:
         self._created = created
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.type_mapping(serializable.helpers.XsdDateTime)
     @serializable.xml_sequence(11)
     def published(self) -> Optional[datetime]:
@@ -1062,7 +1062,7 @@ class Vulnerability:
     def published(self, published: Optional[datetime]) -> None:
         self._published = published
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.type_mapping(serializable.helpers.XsdDateTime)
     @serializable.xml_sequence(12)
     def updated(self) -> Optional[datetime]:
@@ -1078,7 +1078,7 @@ class Vulnerability:
     def updated(self, updated: Optional[datetime]) -> None:
         self._updated = updated
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(13)
     def credits(self) -> Optional[VulnerabilityCredits]:
         """
@@ -1093,10 +1093,10 @@ class Vulnerability:
     def credits(self, credits: Optional[VulnerabilityCredits]) -> None:
         self._credits = credits
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'tool')
     @serializable.xml_sequence(14)
-    def tools(self) -> "SortedSet[Tool]":
+    def tools(self) -> 'SortedSet[Tool]':
         """
         The tool(s) used to identify, confirm, or score the vulnerability.
 
@@ -1109,7 +1109,7 @@ class Vulnerability:
     def tools(self, tools: Iterable[Tool]) -> None:
         self._tools = SortedSet(tools)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_sequence(15)
     def analysis(self) -> Optional[VulnerabilityAnalysis]:
         """
@@ -1124,10 +1124,10 @@ class Vulnerability:
     def analysis(self, analysis: Optional[VulnerabilityAnalysis]) -> None:
         self._analysis = analysis
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'target')
     @serializable.xml_sequence(16)
-    def affects(self) -> "SortedSet[BomTarget]":
+    def affects(self) -> 'SortedSet[BomTarget]':
         """
         The components or services that are affected by the vulnerability.
 
@@ -1140,10 +1140,10 @@ class Vulnerability:
     def affects(self, affects_targets: Iterable[BomTarget]) -> None:
         self._affects = SortedSet(affects_targets)
 
-    @property  # type: ignore[misc]
+    @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'property')
     @serializable.xml_sequence(17)
-    def properties(self) -> "SortedSet[Property]":
+    def properties(self) -> 'SortedSet[Property]':
         """
         Provides the ability to document properties in a key/value store. This provides flexibility to include data not
         officially supported in the standard without having to use additional namespaces or create extensions.
