@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # This file is part of CycloneDX Python Lib
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,11 +15,11 @@
 # SPDX-License-Identifier: Apache-2.0
 # Copyright (c) OWASP Foundation. All Rights Reserved.
 
-import unittest
 from datetime import datetime, timedelta
 from decimal import Decimal
+from typing import Any
 from unittest import TestCase
-from unittest.mock import Mock, patch
+from unittest.mock import patch
 
 from cyclonedx.model import XsUri
 from cyclonedx.model.impact_analysis import ImpactAnalysisAffectedStatus
@@ -36,27 +34,11 @@ from cyclonedx.model.vulnerability import (
     VulnerabilitySeverity,
     VulnerabilitySource,
 )
-
-from .data import reorder
+from tests import reorder, uuid_generator
 
 
 class TestModelVulnerability(TestCase):
 
-    @unittest.skip('Deprecated in Schema Version 1.4')
-    def test_v_rating_scores_empty(self) -> None:
-        vr = VulnerabilityRating()
-        self.assertFalse(vr.score)
-
-    @unittest.skip('Deprecated in Schema Version 1.4')
-    def test_v_rating_scores_base_only(self) -> None:
-        vr = VulnerabilityRating(score_base=1.0)
-        self.assertTrue(vr.score)
-
-    @unittest.skip('Deprecated in Schema Version 1.4')
-    def test_v_rating_scores_all(self) -> None:
-        vr = VulnerabilityRating(score_base=1.0, score_impact=3.5, score_exploitability=5.6)
-        self.assertTrue(vr.score)
-
     def test_v_severity_from_cvss_scores_single_critical(self) -> None:
         self.assertEqual(
             VulnerabilitySeverity.get_from_cvss_scores(9.1),
@@ -186,11 +168,10 @@ class TestModelVulnerability(TestCase):
             'SOMETHING_OR_OTHER'
         )
 
-    @patch('cyclonedx.model.bom_ref.uuid4', return_value='0afa65bc-4acd-428b-9e17-8e97b1969745')
-    def test_empty_vulnerability(self, mock_uuid: Mock) -> None:
+    @patch('cyclonedx.model.bom_ref.uuid4', side_effect=uuid_generator(version=4))
+    def test_empty_vulnerability(self, *_: Any, **__: Any) -> None:
         v = Vulnerability()
-        mock_uuid.assert_called()
-        self.assertEqual(str(v.bom_ref), '0afa65bc-4acd-428b-9e17-8e97b1969745')
+        self.assertEqual(str(v.bom_ref), '00000000-0000-4000-8000-000000000001')
         self.assertIsNone(v.id)
         self.assertIsNone(v.source)
         self.assertFalse(v.references)
@@ -338,8 +319,8 @@ class TestModelBomTargetVersionRange(TestCase):
             BomTargetVersionRange(version='1.0.0'),
             BomTargetVersionRange(version='2.0.0', status=ImpactAnalysisAffectedStatus.AFFECTED),
             BomTargetVersionRange(version='1.0.0', status=ImpactAnalysisAffectedStatus.UNAFFECTED),
-            BomTargetVersionRange(version_range='1.0.0 - 2.0.0', status=ImpactAnalysisAffectedStatus.UNAFFECTED),
-            BomTargetVersionRange(version_range='2.0.0 - 2.1.0', status=ImpactAnalysisAffectedStatus.AFFECTED),
+            BomTargetVersionRange(range='1.0.0 - 2.0.0', status=ImpactAnalysisAffectedStatus.UNAFFECTED),
+            BomTargetVersionRange(range='2.0.0 - 2.1.0', status=ImpactAnalysisAffectedStatus.AFFECTED),
         ]
         sorted_ranges = sorted(ranges)
         expected_ranges = reorder(ranges, expected_order)
