@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # This file is part of CycloneDX Python Lib
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,7 +24,7 @@ from cyclonedx.output.xml import XmlV1Dot3, XmlV1Dot4
 from cyclonedx.schema import OutputFormat, SchemaVersion
 
 
-class TestOutputGeneric(TestCase):
+class TestGetInstanceGeneric(TestCase):
 
     @classmethod
     def setUpClass(cls) -> None:
@@ -34,17 +32,17 @@ class TestOutputGeneric(TestCase):
         cls._bom.components.add(Component(name='setuptools'))
 
     def test_get_instance_default(self) -> None:
-        i = get_instance(bom=TestOutputGeneric._bom)
+        i = get_instance(self._bom)
         self.assertIsInstance(i, XmlV1Dot4)
 
     def test_get_instance_xml_default(self) -> None:
-        i = get_instance(bom=TestOutputGeneric._bom, output_format=OutputFormat.XML)
+        i = get_instance(bom=self._bom, output_format=OutputFormat.XML)
         self.assertIsInstance(i, XmlV1Dot4)
 
     def test_get_instance_xml_v1_3(self) -> None:
-        i = get_instance(bom=TestOutputGeneric._bom, output_format=OutputFormat.XML, schema_version=SchemaVersion.V1_3)
+        i = get_instance(bom=self._bom, output_format=OutputFormat.XML, schema_version=SchemaVersion.V1_3)
         self.assertIsInstance(i, XmlV1Dot3)
 
     def test_component_no_version_v1_3(self) -> None:
-        i = get_instance(bom=TestOutputGeneric._bom, schema_version=SchemaVersion.V1_3)
+        i = get_instance(bom=self._bom, schema_version=SchemaVersion.V1_3)
         self.assertIsInstance(i, XmlV1Dot3)
